#version 430 core
/*
* Copyright 2018 Vienna University of Technology.
* Institute of Computer Graphics and Algorithms.
* This file is part of the ECG Lab Framework and must not be redistributed.

Eva | Adapted according to https://learnopengl.com/Advanced-Lighting/Normal-Mapping
*/

in VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
	vec2 lm_uv;
} vert;

out vec4 color;

uniform vec3 camera_world;

uniform vec3 materialCoefficients; // x = ambient, y = diffuse, z = specular 
uniform float specularAlpha;
uniform sampler2D diffuseTexture;
uniform sampler2D normalTexture;
uniform sampler2D lightmapTexture;
uniform bool isNormalMappingActive; // switch between disabled/enabled normal-mapping on key-toggle

uniform struct DirectionalLight {
	vec3 color;
	vec3 direction;
} dirL;

uniform struct PointLight {
	vec3 color;
	vec3 position;
	vec3 attenuation;
} pointL;


vec3 phong(vec3 n, vec3 l, vec3 v, vec3 diffuseC, float diffuseF, vec3 specularC, float specularF, float alpha, bool attenuate, vec3 attenuation) {
	float d = length(l);
	l = normalize(l);
	float att = 1.0;	
	if(attenuate) att = 1.0f / (attenuation.x + d * attenuation.y + d * d * attenuation.z);
	vec3 r = reflect(-l, n);
	return (diffuseF * diffuseC * max(0, dot(n, l)) + specularF * specularC * pow(max(0, dot(r, v)), alpha)) * att; 
}

void main() {	
	vec3 n = normalize(vert.normal_world);
	vec3 v = normalize(camera_world - vert.position_world);
	
	// load normal texture
	vec3 normalColor = texture(normalTexture, vert.uv).rgb;

	// load diffuse texture
	vec3 texColor = texture(diffuseTexture, vert.uv).rgb * texture(lightmapTexture, vert.lm_uv).rgb;
	
	normalColor = normalize(normalColor * 2.0 - 1.0);
	
	// when true -> using normal texture for lightning
	vec3 normal = isNormalMappingActive ? normalColor : n; 

	// add directional light contribution
	color.rgb += phong(normal, -dirL.direction, v, dirL.color * texColor, materialCoefficients.y, dirL.color, materialCoefficients.z, specularAlpha, false, vec3(0));
			
	// add point light contribution
	color.rgb += phong(normal, pointL.position - vert.position_world, v, pointL.color * texColor, materialCoefficients.y, pointL.color, materialCoefficients.z, specularAlpha, true, pointL.attenuation);
}

